/* programa de comunicaciones al exterior
    mediante un módulo GSM/GPRS SIM900
   Recibe ordenes via RF para enviar mensajes SMS
   y hacer llamadas perdidas a móviles
   Tambien dispone de una rutina para activar Relays
  ============================================================================
  HISTORIA DEL MANTENIMIENTO DEL PROGRAMA (Maintenance log)
  ============================================================================
  2016/06/01 - Modificacion del registro de comunicación por RF
  Añadir 2 campos para indicar el menu y el submenu activos y 2 mas reservados
  para futuro uso
  ============================================================================
  2016/06/02 - Liberación de memoria RAM para evitar situaciones de inestabilidad
  Cambiar Serial.print("literal"); por Serial.print(F("literal")); (= Serial.Println)
  Resultado obtenido:
  pasar de 956 bytes libres a 1.684 bytes libres
  ============================================================================
*/
//---------------------------------------------------------------------------------
// datos globales del programa
// inici definicio de dades internas del programa
char* lastMsg;                 // Guarda el ultimo mensaje enviado por SMS
// Para informacion al cliente cuando lo pida
int conta[6] = {0, 0, 0, 0, 0, 0}; // Tabla contadores, uno por mensaje (i)
int contaTotal       = 0;      // Total de peticiones recibidas
int contaSMS         = 0;      // Total de SMS enviados
int contaPerdidas    = 0;      // Total de llamadas perdidas enviadas
int anteriorPeticion = 0;      // Ultima peticion procesada
unsigned long ahora  = 0;
byte menuActivo      = 0;
byte submenuActivo   = 0;

int esperaTimeOut    = 0;      // Limite de tiempo antes de dar TimeOut
int i                = 6;      // Numero de mensajes "vivos" Actualmente
int x                = 0;      // Codigo de peticion del maestro (1 a i)
int vueltas          = 0;      // limitar las vueltas del loop  // debug
// fi definicio de dades internas del programa

//---------------------------------------------------------------------------------

// inici definicio de dades per GSM/GPRS
// SIM900 Para GPRS
#include <SoftwareSerial.h>
#include <string.h>;            // Librería para manejo de cadenas
SoftwareSerial SIM900(2, 3);

int pinLED           = 7;       // El 13 Lo necesita nRF24L01
int pinRelay         = 6;
int pinActivarGPRS   = 2;       // pin para activar el módulo GPRS

char* msg[] =                   // Tabla de mensajes para enviar por SMS
{
  // A mantener aumentar los sensores
  "MSG1 Alarma temperatura         ",
  "MSG2 Alarma posible inundación  ",
  "MSG3 Intruso infrarrojos        ",
  "MSG4 Intruso ultrasonido        ",
  "MSG5 Intruso movimiento         ",
  "MSG6 Corte electricidad         ",
};
//------------------------------------------------------------------

// Inici definicio llibrerias per al control del temps
#include <TimeLib.h>
#include "Wire.h"
bool estaEnHora = false;
// Fi definicio llibrerias per al control del temps

//------------------------------------------------------------------

// Inici definicio llibrerias i asignacions per RadioFrecuencia
#include <SPI.h>
#include "nRF24L01.h"
#include "RF24.h"

#define pinCE 9               // Pin entrada CE
#define pinCSN 10             // pin sortida CSN
RF24 radio(pinCE, pinCSN);

// Radio pipes per escoltar i emitir respectivament
// Control emiteix pel primer pipe i nosaltres lo escoltem
// Nosaltres emitirem pel segon pipe que es el que escolta control
// Aquest programa de moment no emiteix missatges. Nomes escolta

const uint64_t pipes[2] = { 0xF0F0F0F0E1LL, 0xF0F0F0F0D2LL };
// Fi definicio llibrerias i asignacions per RadioFrecuencia

//------------------------------------------------------------------

// Estructura de dades per a comunicar per RF entre Arduinos
// Recomenat tamany <= 32 Bytes
struct DATOS
{
  byte cap;            // Marca inicio = 123   // Longitud 1 byte acumulada =  1 Byte
  byte sensor;         // Codi numeric sensor  // Longitud 1 byte acumulada =  2 Byte
  // 1 - sensor de clima      DTH11
  // 2 - sensor d'aigua       FC-37
  // 3 - sensor d'infrarojos  L-IR
  // 4 - sensor d'Ultrasons   HC-SR04
  // 5 - sensor de Moviment   HC-SR501
  // 6 - detector tallCorrent Transformador + alimentador adaptat al efecte
  // 0 - aquest valor indica que es tracta d'un registre de sincronisme
  //     de data i hora que ve informat al camp 'epoch'
  byte serie;          // Nº sequencia sensor   // Longitud 1 byte acumulada = 3 Byte
  int valor1;          // Sensor temperatura    // Longitud 4 byte acumulada = 7 Byte
  int valor2;          // Sensor infrarojos    // Longitud 4 byte acumulada = 11 Byte
  float valor3;        // Sensor moviment     // Longitud 4 byte acumulada = 15 Byte
  int valor4;          // Sensor ultrasons    // Longitud 2 byte acumulada = 17 Byte
  byte valor5;         // Sensor de corrent    // Longitud 1 byte acumulada = 18 Byte
  byte interrupcio;    // Flag Polling  = 111   // Longitud 1 byte acumulada = 19 Byte
                       // Flag Interrup = 222
  byte alarma;         // Codi d'alarma         // Longitud 1 byte acumulada = 20 Byte
  unsigned long epoch; // Hora UNIX             // Longitud 4 byte acumulada = 24 Byte
  byte periferic;      // Arduino que contesta  // Longitud 1 byte acumulada = 25 Byte
  byte menuActivo;     // Menu seleccionado     // Longitud 1 byte acumulada = 26 Byte
  byte submenuActivo;  // submenu seleccionado  // Longitud 1 byte acumulada = 27 Byte
  byte reservado1;     // Reservado futuro uso  // Longitud 1 byte acumulada = 28 Byte
  byte reservado2;     // Reservado futuro uso  // Longitud 1 byte acumulada = 29 Byte
  byte cua;            // Marca final = 234     // Longitud 1 byte acumulada = 30 Byte
};
union REDATOS {        // Redefinicion de la estructura DATOS
  DATOS dato;
  byte  b[sizeof(DATOS)];
}
MIS_DATOS, TUS_DATOS; // Dues areas independients iguals per a emitir i rebre dades
// Fi definicions estructures de dades RadioFrecuencia

//------------------------------------------------------------------

void setup()
{
  //pinMode(pinCSN, OUTPUT);
  Serial.begin(115200);

  // Inici Setup i configuració RF NRF24L01
  radio.begin();
  radio.setRetries(15, 15);            // esperas entre reintentos
  radio.setPayloadSize(32);            // Tamany del lot que sembla mes fiable
  radio.setChannel(106);               // De 0 a 127
  radio.setDataRate(RF24_250KBPS);     // reducir ancho de banda

  // radio.setPALevel(RF24_PA_MIN);    // PA Level minimo
  // radio.setPALevel(RF24_PA_LOW);    // PA Level bajo
  radio.setPALevel(RF24_PA_HIGH);   // PA Level alto
  // radio.setPALevel(RF24_PA_MAX);    // PA Level maximo (defecto)

  // Engegar la radio

  // Control emiteix pel primer pipe i nosaltres lo escoltem
  // Nosaltres emitirem pel segon pipe que es el que escolta control

  radio.openWritingPipe(pipes[1]);     // Aquesta versió no parla
  radio.openReadingPipe(1, pipes[0]);  // nomes escolta
  radio.startListening();
  // Fi Setup i configuració RF NRF24L01

  // Inici Setup i configuració GPRS
  pinMode(pinLED, OUTPUT);
  pinMode(pinActivarGPRS, OUTPUT);
  //pinMode(pinRelay, OUTPUT);         // Actualment no hi ha relay
  //digitalWrite(pinRelay, HIGH); 

  //  Para GPRS
  SIM900.begin(19200);
  activarModuloGPRS();
  delay(20000);                       // Dar tiempo al logon en la red. Es lento en arrancar.
  // Fi Setup i configuració GPRS

  // HAcer acto de presencia en bus Serial para DEBUG
  Serial.println(F("Bloque de comunicaciones GSM/GPRS"));
  Serial.println(F("Dando servicios de SMS y Perdidas a Moviles:"));
  Serial.println(F("====================================="));
  Serial.println(F("Esperando sincronia"));
  Serial.println(F("====================================="));

  // sincronizamos el reloj

  ahora = now();
  while ((! estaEnHora) || (ahora < 1000000))
  {
    Serial.println(F("Esperando Sincronia-------------------"));
    if (leerMensaje());                 // Si la respuesta es true se ha leido correctamente
    {
      if (TUS_DATOS.dato.sensor == 0)   // El registro de sincronismo no requiere contestacion
        sincronizaArduino();
    }
    delay(100);                         // esperamos 1/10 de segundo y reintentamos
  }
  Serial.println(F("====================================="));
  time_t t = now();
  printFecha(t);
  Serial.println(F("Sincronizado y Funcionando"));
  Serial.println(F("====================================="));
}

//---------------------------------------------------------------------------------

void loop(void)
{
  if (leerMensaje()) // Leeremos todo lo que circula por el pipe de entrada
  {
    // El missatge ha arribat integrament
    // Ara hem de decidir si es per nosaltres o no
    // Aquest programa tracta nomes els registres de alarma
    // ((sensor = 111) && ( alarma != 0)) // Alarmes de "Polling"
    // ((sensor = 222) && ( alarma != 0)) // Alarmes d'Interrupció
    // i el registre de sincronisme (sensor = 0)

    if (TUS_DATOS.dato.sensor == 0)        // Registre de sincronia
      sincronizaArduino();
      
    else if ((TUS_DATOS.dato.sensor == ( 111 || 222)) && ( TUS_DATOS.dato.alarma != 0)) // Registre d'alarma
      procesarPeticion(TUS_DATOS.dato.alarma );
    else
      Serial.println(F("Registro no tratado en este nodo"));
  }
}

//------------------------------------------------------------------

bool leerMensaje()
{
  bool result = false;
  // Esperamos hasta tener una respuesta o un TimeOut
  unsigned long tiempoTimeOut = millis();
  bool timeout = false;

  while ( ! radio.available() && ! timeout )
    if (millis() - tiempoTimeOut > esperaTimeOut) timeout = true;
  // Ahora, tenemos datos para leer o TimeOut

  if (timeout) // No tenemos datos para leer recuperar. TimeOut
  {
    result = false;
    Serial.println(F("Respuesta TimeOut -----------------------------"));
  }
  else
  {
    // descargamos bloques hasta el final del mensaje
    // a destacar que el mensaje que manejamos es de 27 bytes
    // y tenemos radio.setPayloadSize(32)
    // Es decir que el mensaje cabe en un solo bloque

    bool done = false;
    while (!done)
      done = radio.read( &TUS_DATOS, sizeof(DATOS) );
    Serial.println(F("Mensaje leido-----------------------------"));
    Serial.print(F("cap = "));
    Serial.print(TUS_DATOS.dato.cap);
    Serial.print(F(" cua = "));
    Serial.println(TUS_DATOS.dato.cua);

    // Vemos si se ha leido integramente
    if (TUS_DATOS.dato.cap == 123 && TUS_DATOS.dato.cua == 234)
    {
      result = true;
      Serial.println(F("Respuesta recibida integra -------------------"));
      print_TUS_DATOS();
    }
    else
    {
      result = false;
      Serial.println(F("Lectura fallida. Respuesta Incompleta ---------"));
      print_TUS_DATOS();
    }
  }
  return result;
}

//---------------------------------------------------------------------
/* Se discutirá en el futuro si conviene una contestación que tardaría
    varios segundos en producirse
  //---------------------------------------------------------------------
  void preparaContestacion()   // copiar TUS_DATOS a MIS_DATOS i preparar codis
  {
  for (int j = 0 ; j < sizeof(DATOS) ; j++)  // Copiar TUS:DATOS a MIS_DATOS
    MIS_DATOS.b[j] = TUS_DATOS.b[j];

  // cap = 234 i cua = 123 para indicar al peticionario que es una alarma
  // y que ya ha sido procesada. a fin de que no repita le petición
  // aunque las peticiones repetidas se controlan tambien aqui

  MIS_DATOS.dato.cap = 234;
  MIS_DATOS.dato.epoch = now();
  MIS_DATOS.dato.cua = 123;
  }

  //---------------------------------------------------------------------

  void contestar()                   // contesta a control
  {
  radio.stopListening();           // Cambio de escucha a emision

  // Reintentamos el envio hasta ok o TimeOut (10000ms)
  //------------------------------------------------------------------
  // Hemos de esperar el tiempo que haga falta y luego lo mediremos
  // DEBUG
  unsigned long tiempoTimeOut = millis();
  unsigned long esperado;          // Para medir el coste en tiempo
  //------------------------------------------------------------------
  bool timeout = false;
  bool ok = false;

  // Intentamos emitir hasta ok o TimeOut (10000ms)
  while ((!ok) && (!timeout))
    if (millis() - tiempoTimeOut > 10000) timeout = true;

  esperado = millis() - tiempoTimeOut;
  // Ahora, hay envio correcto o TimeOut

  if (timeout)
    Serial.println(F("NO ok...ERROR emisión. Respuesta NO enviada ------"));
  else
  {
    Serial.println(F("ok...Respuesta enviada correctamante--------------"));
    Serial.print(F("Tiempo invertido-millis = ")); Serial.println(esperado);
  }
  // Cambio de emision a escucha
  radio.startListening();
  }
*/
//---------------------------------------------------------------------
void procesarPeticion(int x)
{
  if (x < 1 || x > i)
  {
    // Actualmente solo se contemplan seis mensajes (i = 6)
    Serial.print(F("Valor de x invalido x=")); Serial.println(x);  // test
  }
  else     // Tenemos x en el rango
  {
    Serial.print(F("Valor de x correcto x = ")); Serial.println(x);  // test
    contaTotal++;            // Registra numero total de peticiones

    // Se envia SMS solamente la primera vez que se solicita cada mensaje
    // Si hay mas peticiones se envian hasta DOS llamadas perdidas y luego nada

    if (conta[x - 1] == 0)    // Si primera vez  --> Se envia SMS de alarma
    {
      // Hay varias rutinas diferentes para probar. Parece ser que funcionan TODAS

      llamadaPerdida();  // para DEBUG no enviaremos SMS. Haremos llamadas perdidas
      // *******************************************************
      // sendSMS();         // Enviar SMS // En real, una de estas tres
      ////// mensaje_SMS();  // Enviar SMS
      ////// mensajeSMS();   // Enviar SMS
      lastMsg = msg[x];  // Guarda el ultimo mensaje enviado
      conta[x - 1]++;    // Incrementa la cuenta del SMS[x] concreto enviado
      contaSMS++;        // Incrementa la cuenta total de SMS's enviados
    }
    else if (conta[x - 1] < 3)    // Si el contador < 3 --> Se envia llamada perdida (2 y 3)
    {
      llamadaPerdida();  // Alarmas dos y tres iguales, luego nada.
      contaPerdidas++;   // Incrementa la cuenta de llamadas perdidas
    }
    else                          // No se envia nada mas
    {
      lastMsg = "SMS repetido, NO enviado";
      anteriorPeticion = x;
    }
  }
  anteriorPeticion = x;
  Serial.print(F("x = ")); Serial.print(x);
  Serial.print(F(" Mensaje enviado = ")); Serial.println(msg[x]);
}

//---------------------------------------------------------------------

void sincronizaArduino()
{
  ahora = now();
  if ((!estaEnHora) || ( ahora < 1000000)) // Si el temps encara no es ajustat
  {
    setTime(TUS_DATOS.dato.epoch);
    ahora = now();
    estaEnHora = true;
    // Ademas de sincronizar el reloj, recojemos información sobre menu activo
    menuActivo = TUS_DATOS.dato.menuActivo;
    submenuActivo = TUS_DATOS.dato.submenuActivo;
  }
}


//----------------------------------------------------------------------------------
/* No se invoca de momento
  void actuaRelay() { // De momento el relay parpadea x veces
  for ( int j = 0 ; j < x ; j++ ) { // y parpadea x veces para marcar peticion x
    digitalWrite(pinRelay, LOW);
    delay(2000);
    digitalWrite(pinRelay, HIGH);
    delay(2000);
  }
  }
*/
//----------------------------------------------------------------------------------
/* Rutina alternativa a sendSMS
  void mensajeSMS () { //Funcion para mandar mensaje de texto
  SIM900.print("AT+CMGF=1\r")); // Comando AT que pone modo texto
  delay(1000);
  SIM900.println("AT + CMGS = \"+34692825542\"")); // Movil al que se envia el mensaje
  SIM900.println("Arduino te saluda")); // message to send
  delay(100);
  SIM900.println((char)26); // End AT command with a ^Z, ASCII code 26 //Comando de finalizacion
  delay(100);
  SIM900.println();
  delay(5000); // Tiempo para que se envie el mensaje
  Serial.println(F("SMS enviado con exito"));
  }

  //----------------------------------------------------------------------------------
  /* Rutina alternativa a sendSMS
  void mensaje_sms() { //Funcion para mandar mensaje de texto
  SIM900.print("AT+CMGF=1\r")); // Comando AT que pone modo texto
  delay(100);
  SIM900.println("AT + CMGS = \"+34692825542\"")); // Movil al que se envia el mensaje
  SIM900.println("Arduino te saluda")); // message to send
  delay(100);
  SIM900.println((char)26); // End AT command with a ^Z, ASCII code 26 //Comando de finalizacion
  delay(100);
  SIM900.println();
  delay(5000); // Tiempo para que se envie el mensaje
  Serial.println(F("SMS enviado con exito"));
  }
*/
//----------------------------------------------------------------------------------

void sendSMS()
{
  // Serial.begin(19200); //velocidad de conexión con módulo GPRS
  Serial.println(F("AT+IPR=19200"));
  activarModuloGPRS(); //activar el módulo GPRS
  delay(10000); //delaymos 10 segundos a que se active el módulo GPRS
  Serial.println(F("AT+CMGF=1")); //activamos el modo SMS
  delay(1500);
  Serial.print(F("AT+CMGS=")); // comando AT para enviar SMS
  Serial.print((char)34);   // enviar caracter " = 34 dec
  Serial.print(F("692825542")); // numero de movil receptor del SMS
  Serial.println((char)34);   // enviar caracter " = 34 dec
  delay(1500);
  Serial.println(msg[i]); // texto del SMS
  delay(500);
  Serial.print((char)0x1A); // final del SMS, comando 1A (hex)
  delay(10000);
  Serial.println(F("AT*PSCPOF"));  // desactivar el módulo GPRS/*
}

//----------------------------------------------------------------------------------

void llamadaPerdida() // Función que permite llamar a un celular local
{
  activarModuloGPRS();
  // SIM900.println(F("ATD 10725275;")); //Numero local
  SIM900.println(F("ATD 692825542;"));  //Celular
  delay(100);
  SIM900.println();
  delay(30000); // wait for 30 seconds...
  SIM900.println(F("ATH")); // Cuelga el telefono
  delay(1000);
}

//----------------------------------------------------------------------------------

//procedimiento para activar el módulo GPRS
void activarModuloGPRS()
{
  Serial.println(F("AT+IPR=19200"));
  digitalWrite(pinActivarGPRS, LOW);
  delay(2000);
  digitalWrite(pinActivarGPRS, HIGH);
  delay(2000);
  digitalWrite(pinActivarGPRS, LOW);
  delay(2000);
}

//----------------------------------------------------------------------------------
/*
  void listarEstructura() {
  Serial.print(F("Last message = ")); Serial.println(lastMsg);
  Serial.print(F("Contador 0 = ")); Serial.println(conta[0]);
  Serial.print(F("Contador 1 = ")); Serial.println(conta[1]);
  Serial.print(F("Contador 2 = ")); Serial.println(conta[2]);
  Serial.print(F("Contador 3 = ")); Serial.println(conta[3]);
  Serial.print(F("Contador 4 = ")); Serial.println(conta[4]);
  Serial.print(F("Contador 5 = ")); Serial.println(conta[5]);
  //  for ( int j = 0 ; j < 6 ; j++ ) {
  //    Serial.print(F("Contadores")); Serial.println(conta[j]);
  //  }
  Serial.print(F("Conta Total = ")); Serial.println(contaTotal);
  Serial.print(F("Conta SMS = ")); Serial.println(contaSMS);
  Serial.print(F("Anterior Peticion = ")); Serial.println(anteriorPeticion);
  Serial.println(F("========================================"));
  }
*/
//---------------------------------------------------------------------

void print_TUS_DATOS() // Debug
{
  time_t t = now();
  printFecha(t);
 Serial.print(F("cap             = ")); Serial.println(TUS_DATOS.dato.cap);
  Serial.print(F("sensor          = ")); Serial.println(TUS_DATOS.dato.sensor);
  Serial.print(F("serie           = ")); Serial.println(TUS_DATOS.dato.serie);
  Serial.print(F("valor sensor 1  = ")); Serial.println(TUS_DATOS.dato.valor1);
  Serial.print(F("valor sensor 2  = ")); Serial.println(TUS_DATOS.dato.valor2);
  Serial.print(F("valor sensor 3  = ")); Serial.println(TUS_DATOS.dato.valor3);
  Serial.print(F("valor sensor 4  = ")); Serial.println(TUS_DATOS.dato.valor4);
  Serial.print(F("valor sensor 5  = ")); Serial.println(TUS_DATOS.dato.valor5);
  Serial.print(F("alarma          = ")); Serial.println(TUS_DATOS.dato.alarma);
  Serial.print(F("interrupcio     = ")); Serial.println(TUS_DATOS.dato.interrupcio);
  Serial.print(F("epoch           = ")); Serial.println(TUS_DATOS.dato.epoch);
  Serial.print(F("periferic       = ")); Serial.println(TUS_DATOS.dato.periferic);
  Serial.print(F("menuActivo      = ")); Serial.println(TUS_DATOS.dato.menuActivo);
  Serial.print(F("submenuActivo   = ")); Serial.println(TUS_DATOS.dato.submenuActivo);
  Serial.print(F("reservado1      = ")); Serial.println(TUS_DATOS.dato.reservado1);
  Serial.print(F("reservado2      = ")); Serial.println(TUS_DATOS.dato.reservado2);
  Serial.print(F("cua             = ")); Serial.println(TUS_DATOS.dato.cua);
  Serial.println(F("================================================="));
}

//---------------------------------------------------------------------

void printFecha(time_t  t) // Debug
{
  Serial.print(F("Fecha y Hora (D/M/A H;M;S) -->  "));
  Serial.print(day(t)) ;    Serial.print(+ "/") ;   Serial.print(month(t));   Serial.print(+ "/") ;
  Serial.print(year(t));    Serial.print( " ") ;
  Serial.print(hour(t));   Serial.print(+ ":") ;   Serial.print(minute(t));   Serial.print(F(":")) ;
  Serial.println(second(t));
  delay(500);
}

//---------------------------------------------------------------------
/*
  void espera(int t)
  {
  // Rutina para substituir delays no deseados
  t = t - ((t / 1000) * 1.7); // Compensacion por incremento de tiempo observado con respecto al delay();
  int coci = t / 1000;
  int rest = t % 1000;
  while (coci > 0) {
    unsigned long tempsini = millis();
    unsigned long tempsfin = tempsini;
    while ((tempsfin - tempsini) <= 1000)  { // loop de 1000ms
      tempsfin = millis();
    }
    coci = coci - 1;
  }
  while (rest > 0) {
    delayMicroseconds(1000); // delay Mil microsegons (un milisegon))
    rest = rest - 1;         // cada volta
  }
  }
*/
//----------------------------------------------------------------------------------

//Fin del codigo/////////////////////////////////////////////////////////////////////////////////////////////
