/* Programa model per generar programes per a sensors
   =====================================================
   Si aquest programa NO ES DIU "modeloSensor"
   es tracta de un programa generat per copia i posterior
   parametrització orientada a un sensor particular
   =====================================================
   Si es diu "modeloSensor" es tracta del model original.
   Per tal de generar un programa per controlar un sensor
   determinat cal:
   1) Copiar aquest font amb el nom adient
   2) Parametritzar la copia com es detalla a continuació.
   =====================================================
   En comptes de fer tants programes diferents com sensors tenim,
   podem fer servir variacións realment petites d'aquest programa
   per generar d'altres programes especifics per a cadascun dels
   sensors que utilitzem.
   Per aconseguir aixó nomes hem de seguir les instruccions seguents:

   //================================================//
   // Informar els punts asenyalats amb  "MODIFICAR" //
   //                                     =========  //
   //================================================//

    Pretenem amb aquest sistema minimitzar el manteniment del projecte
    mantenint nomes UN PROGRAMA sensor( aquest mateix), UN PROGRAMA de
    control i UN PROGRAMA de comunicació.

   Aquest programa és un model que serveix per generar programes
   que controlin diferents sensors anb una petita intervenció.
   conté definicions i processos per els diferents sensors del projecte
   que poden ser facilment seleccionats generant programes individuals
   per controlar uno o mes sensors.Anb aquesta manera
 *************************************************************************

  Aquest programa rep missatges per RF enviats pel programa 'Control'
  demanant dades de un determinat sensor.

  La actuació del programa 'sensor' ha de ser la següent:

  1) comprovar que el missatge ha estat rebut íntegrament, en cas
  contrari no el tractarà ni el respondrà i deixarà que 'control'
  torni a intentar la comunicació (actualment hi ha 15 intents).

  2) Identificar el codi del missatge rebut i decidir si li pertany.
  En el cas de que el sensor concret identificat li pertanyi, serà
  tractat pel programa informant el valor demanat i tornarà el missatge
  al programa 'control' afegint senyals que permetin verificar la
  recepció complerta del missatge.

  3) El programa llegirá els valors recollits pels sensors demanats i
  informará control del resultats. Si el valor recollit mereix missatge
  d'alarma, s'informará el registre de resposta com cal.

  4) En el cas dels sensors d'interrupció (Infrarojos, Ultrasons i Moviment)
  la execució de les rutines "attached" informará un camp com a senyal, que
  será tractat a la primera demanda de control.

  El programa 'control' emet al començament i cada minut un missatge amb un codi
  preestablert i el temps en format UNIX. Tots i cadascun dels programes dels
  perifèrics lo faran servir com a sincronisme per posar el rellotge en hora.
  Aquest tipus de missatge NO SERA RESPOST per cap dels programes dels Arduinos
  perifèrics ja que no pertany a ningu.

  La resta dels missatges escoltats, son per altres Arduinos i per tant seran ignorats.

  A continuació donem una llista dels sensors actuals del projecte (que podrà créixer en el futur)
  i la seva codificació, asi com també el tipus de tractament associat a cadascun:

  ---Sensor---  Codi  Tractament
  ============  ====  ==========
  Temperatura     1   Polling
  Humitat         2   Polling
  Infrarojos      3   Interrupcio
  Ultrasons       4   Interrupcio
  Moviment        5   Interrupcio
  Tall corrent    6   Polling

  NOMENCLATURA DELS PROGRAMES DE PERIFERICS:
  ==========================================
  1) La paraula 'sensor'
  2) A continuació una referència per cada sensor (tipusSensor).
     (posarem nomes un sensor per Arduino perifèric)
   '_' Un guió baix
   Digit nº de serie del mateix tipus

  Exemples de noms de programes en funció del equipament del Arduino
  ==================================================================
  sensorClima_1 tracta el sensor 1 (clima)     serie 1 (El primer)
                (aquest sensor torna temperatura, humitat i indexCalor)
  sensorAigua_1 tracta el sensor 2 (Aigua)     serie 1 (El primer)
  sensorIR_1 tracta el sensor    3 (Infrarojos)serie 1 (El primer)
  sensorUS_1 tracta el sensor    4 (Ultrasons) serie 1 (El primer)
  sensorUS_2 tracta el sensor    4 (Ultrasons) serie 2 (El segon)
  sensorMO_1 tracta el sensor    5 (Moviment)  serie 1 (El segon)
  sensorVolts_1 tracta el sensor 6 (Tall corrent) serie 1 (l'unic)

  Taula dels pins de plaques Arduino que poden tractar interrupcions
  ==================================================================
  // PLACA         |  PIN Digital asociat als Tipus d'interrupció  |
  // ARDUINO       | INT 0 | INT 1 | INT 2 | INT 3 | INT 4 | INT 5 |
  //----------------------------------------------------------------
  // ARDUINO UNO   | PIN 2 | PIN 3 |   -   |   -   |   -   |   -   |
  // ARDUINO NANO  | PIN 2 | PIN 3 |   -   |   -   |   -   |   -   |
  // ARDUINO MEGA  | PIN 2 | PIN 3 | PIN 21| PIN 20| PIN 19| PIN 18|
  //----------------------------------------------------------------

  Tot i que podem posar mes de un sensor per Arduino periféric, no ho farem si no es tracta
  de una necesitat especifica. Posarem un sol sensor per cada Arduino perifèric.

  Nota ---------------------------------------------------------------------------
  Els programes de perifèrics no faran servir la instrucció 'delay();' degut a que
  potser algun haurà de detectar incidències que nomes es poden tractar amb les
  tractaments d'interrupcions. Faran servir en comptes la nostra rutina 'espera();'.
  ============================================================================
  HISTORIA DEL MANTENIMIENTO DEL PROGRAMA (Maintenance log)
  ============================================================================
  2016/06/01 - Modificacion del registro de comunicación por RF
  Añadir 2 campos para indicar el menu y el submenu activos y 2 mas reservados
  para futuro uso
  ============================================================================
  ============================================================================
  2016/06/02 - Liberación de memoria RAM para evitar situaciones de inestabilidad
  Cambiar Serial.print("literal"); por Serial.print(F("literal")); (= Serial.Println)
  Resultado obtenido:
  pasar de 956 bytes libres a 1.684 bytes libres
  ============================================================================
*/
//------------------------------------------------------------------
// Definicio llibrerias
#include <TimeLib.h>     // Tiempo     

// RadioFrecuencia
#include <SPI.h>
#include "nRF24L01.h"
#include "RF24.h"

#define pinCE 9             // Pin entrada CE
#define pinCSN 10           // pin sortida CSN
RF24 radio(pinCE, pinCSN);
const uint64_t pipes[2] = { 0xF0F0F0F0E1LL, 0xF0F0F0F0D2LL };
// Radio pipes per escoltar i emitir respectivament
// Control emiteix pel primer pipe i nosaltres lo escoltem
// Nosaltres emitirem pel segon pipe que es el que escolta control
// Fi definicio llibrerias i asignacions
//------------------------------------------------------------------

// Estructura de dades per a comunicar per RF entre Arduinos
// Recomenat tamany <= 32 Bytes
struct DATOS
{
  byte cap;            // Marca inicio = 123   // Longitud 1 byte acumulada =  1 Byte
  byte sensor;         // Codi numeric sensor  // Longitud 1 byte acumulada =  2 Byte
  // 1 - sensor de clima      DTH11
  // 2 - sensor d'aigua       FC-37
  // 3 - sensor d'infrarojos  L-IR
  // 4 - sensor d'Ultrasons   HC-SR04
  // 5 - sensor de Moviment   HC-SR501
  // 6 - detector tallCorrent Transformador + alimentador adaptat al efecte
  // 0 - aquest valor indica que es tracta d'un registre de sincronisme
  //     de data i hora que ve informat al camp 'epoch'
  byte serie;          // Nº sequencia sensor   // Longitud 1 byte acumulada = 3 Byte
  int valor1;          // Sensor temperatura    // Longitud 4 byte acumulada = 7 Byte
  int valor2;          // Sensor infrarojos    // Longitud 4 byte acumulada = 11 Byte
  float valor3;        // Sensor moviment     // Longitud 4 byte acumulada = 15 Byte
  int valor4;          // Sensor ultrasons    // Longitud 2 byte acumulada = 17 Byte
  byte valor5;         // Sensor de corrent    // Longitud 1 byte acumulada = 18 Byte
  byte interrupcio;    // Flag Polling  = 111   // Longitud 1 byte acumulada = 19 Byte
                       // Flag Interrup = 222
  byte alarma;         // Codi d'alarma         // Longitud 1 byte acumulada = 20 Byte
  unsigned long epoch; // Hora UNIX             // Longitud 4 byte acumulada = 24 Byte
  byte periferic;      // Arduino que contesta  // Longitud 1 byte acumulada = 25 Byte
  byte menuActivo;     // Menu seleccionado     // Longitud 1 byte acumulada = 26 Byte
  byte submenuActivo;  // submenu seleccionado  // Longitud 1 byte acumulada = 27 Byte
  byte reservado1;     // Reservado futuro uso  // Longitud 1 byte acumulada = 28 Byte
  byte reservado2;     // Reservado futuro uso  // Longitud 1 byte acumulada = 29 Byte
  byte cua;            // Marca final = 234     // Longitud 1 byte acumulada = 30 Byte
};
union REDATOS {        // Redefinicion de la estructura DATOS
  DATOS dato;
  byte  b[sizeof(DATOS)];
}
MIS_DATOS, TUS_DATOS;

//------------------------------------------------------------------

byte periferic        = 5; // Modul Moviment
byte sensor           = 5;
byte serie            = 1;
bool estaEnHora       = false;
unsigned long ahora   = 0;
byte menuActivo       = 0;
byte submenuActivo    = 0;
int esperaTimeOut     = 100;
// Sensores a tratar o no por cada programa particular "sensor_xy..."
bool sensorClima      = false;
bool sensorAigua      = false;
bool sensorInfrarojos = false;
bool sensorUltrasons  = false;
bool sensorMoviment   = false;
bool sensorVolts      = false;

//------------------------------------------------------------------

// Contadores de alarma emitidas (mensaje) de los diferentes sensores
// Cada programa sensor usara la/s suya/s evidentemente
byte alarmas         = 3;  // Numero maximo permitido de cada tipo de alarma
//---------------------------------------------------------------------------
byte maxTemperatura  = 23; // Temperatura maxima. Si se supera saltara alarma
byte contaClima      = 0;  // Contadores de las alarmas emitidas de cada tipo
byte contaAigua      = 0;
byte contaInfrarojos = 0;
byte contaUltrasons  = 0;
byte contaMoviment   = 0;
byte contaVolts      = 0;

//------------------------------------------------------------------

void setup(void)
{
  // MODIFICAR LOS CUATRO VALORES SIGUIENTES
  // =======================================
  // INFORMAR EL Nº DEL periferico (1 a 255)
  periferic = 5; // Poner Nº de periferico
  // INFORMAR EL Nº DEL sensor (actualmente 1 a 6)
  sensor    = 5; // Poner Nº sensor (ver abajo)
  // INFORMAR EL Nº DE  serie (1 a 255)
  serie     = 1; // Poner Nº serie
  // INFORMAR EL SENSOR ACTIVO (DESCOMENTANDO LA LINEA   Nº sensor
  // =================================================== =========
  // sensorClima      = true; // descomentar para activar    1
  // sensorAigua      = true; // descomentar para activar    2
  // sensorInfrarojos = true; // descomentar para activar    3
  // sensorUltrasons  = true; // descomentar para activar    4
  sensorMoviment   = true; // descomentar para activar    5
  // sensorVolts      = true; // descomentar para activar    6
  // ===================================================
  // MODIFICAR - FINAL DE LAS MODIFICACIONES REQUERIDAS
  // ===================================================

  // Activem tots els elements del/s sensor/s triat/s

  //---------------------------------------------------------------

  if (sensorMoviment == true)
  {
    // Declaració variables 'void getMoviment()' (Rutina sensor HC-SR501)

#define pinMoviment 3    // pin  digital del sensor HC-SR501
#define pinLedMoviment 2 // pinLed rutinas d' interrupció tipus '0'

    pinMode (pinMoviment, INPUT);
    attachInterrupt(0, intMoviment, CHANGE);  // executa al loop 'void intMoviment'
  }

  //------------------------------------------------------------------
  
  Serial.begin(115200);

  // Inici SetUp RadioFrecuencia
  //  pinMode(pinCSN, OUTPUT);

  // Setup i configuració RF NRF24L01
  radio.begin();
  radio.setRetries(15, 15);            // esperas entre reintentos
  radio.setPayloadSize(32);            // Tamany del lot que sembla mes fiable
  radio.setChannel(106);               // De 0 a 127
  radio.setDataRate(RF24_250KBPS);     // reducir ancho de banda

  // radio.setPALevel(RF24_PA_MIN);  // PA Level minimo
  // radio.setPALevel(RF24_PA_LOW);  // PA Level bajo
  radio.setPALevel(RF24_PA_HIGH); // PA Level alto
  // radio.setPALevel(RF24_PA_MAX);  // PA Level maximo (defecto)
  
  // Engegar la radio

  // Control emiteix pel primer pipe i nosaltres lo escoltem
  // Nosaltres emitirem pel segon pipe que es el que escolta control
  radio.openWritingPipe(pipes[1]);
  radio.openReadingPipe(1, pipes[0]);
  radio.startListening();

  // Fi Setup i configuració RF NRF24L01

  // HAcer acto de presencia en bus Serial para DEBUG
  Serial.print(F("Periferico Nº ")); Serial.print(periferic);
  Serial.println(F(" Dando servicios de:"));
  if (sensorMoviment == true) Serial.println(F("Moviment"));
  Serial.println(F("====================================="));
  Serial.println(F("Esperando sincronia"));
  Serial.println(F("====================================="));

  // Sincronizamos primero porque con los sensores de interrupción
  // no podemos distraer al procesador escuchando la radio
  // La espera media son 30 segundos, ya que control emite un
  // registro de sincronismo cada minuto.
  ahora = now();
  while ((! estaEnHora) || (ahora < 1000000))
  {
    //Serial.println(F("Esperando Sincronia-------------------"));
    if (leerMensaje());                 // Si la respuesta es true se ha leido correctamente
    {
      if (TUS_DATOS.dato.sensor == 0)   // El registro de sincronismo no requiere contestacion
        sincronizaArduino();
    }
    //espera(100);                        // esperamos 1/10 de segundo y reintentamos
  }
  Serial.println(F("====================================="));
  time_t t = now();
  printFecha(t);
  Serial.println(F("Sincronizado y Funcionando"));
  Serial.println(F("====================================="));

  if (sensor == 3 || sensor == 4 || sensor == 5)  // Sensors d'Interrupcio
  {
    // Un cop acabat de sincronitzar invertim les pipes
    // Aquestes rutines emetran missatges pel primer pipe
    // Donat que GSM/GPSR n'ha de escoltar control i interrupcions
    radio.openWritingPipe(pipes[0]);
    radio.openReadingPipe(1, pipes[1]);
    radio.startListening();         // Per reutilitzar les mateixas rutines
  }
}
/**********************************************************************
   Nota importante:
   Debido a la enorme velocidad de las transmisiones por RF
   NO SE PUEDEN poner espera() ni delay() entre emision y escucha
 *********************************************************************/

//---------------------------------------------------------------------

void loop(void)
{
  // Activamos las rutinas del loop en funcion del sensor
  // Si el programa controla un sensor de interrupcion queda
  // pendiente del mismo hasta que esta   MIS_DATOS.dato.sensorIR = 70;se produzca.

  esperaTimeOut = 100;

  if (sensor == 5)
  getMoviment();
  
  // Si el programa controla un sensor de polling queda
  // pendiente de de la RF hasta tener una peticion

  if (leerMensaje());                // Si la respuesta es true se ha leido correctamente
  {
    // Registro de sincronismo ---------------------------------
    if (TUS_DATOS.dato.sensor == 0) // El registro de sincronismo no requiere contestacion
      sincronizaArduino();

    // Investigamos el registro leido para ver si nos corresponde tratarlo
    else if (TUS_DATOS.dato.sensor == sensor         // nuestro sensor
             && TUS_DATOS.dato.serie == serie        // serie de nuestro sensor
             && TUS_DATOS.dato.interrupcio == 0)    // No es registro de interrupcion
    {

    }
    // Si no nos corresponde, se ignora
    //else
    //  Serial.println(F("Registro no tratado en este nodo")); // DEBUG
  }
}

//---------------------------------------------------------------------

bool leerMensaje()
{
  bool result = false;
  // Esperamos hasta tener una respuesta o un TimeOut
  unsigned long tiempoTimeOut = millis();
  bool timeout = false;

  while ( ! radio.available() && ! timeout )
    if (millis() - tiempoTimeOut > esperaTimeOut) timeout = true;

  // Ahora, tenemos datos para leer o TimeOut

  if (timeout) // No tenemos datos para leer recuperar. TimeOut
  {
    result = false;
    //Serial.println(F("Lectura TimeOut -----------------------------"));
  }
  else
  {
    // descargamos bloques hasta el final del mensaje
    // a destacar que el mensaje que manejamos es de 27 bytes
    // y tenemos radio.setPayloadSize(32)
    // Es decir que el mensaje cabe en un solo bloque

    bool done = false;
    while (!done)
      done = radio.read( &TUS_DATOS, sizeof(DATOS) );
    // -----------------------------------------------------DEBUG
    //Serial.println(F("Mensaje leido------------------------------"));
    //Serial.print(F("cap = "));
    //Serial.println(TUS_DATOS.dato.cap);
    //Serial.print(F("cua = "));
    //Serial.println(TUS_DATOS.dato.cua);
    // Vemos si se ha recibido integramente
    if (TUS_DATOS.dato.cap == 123 && TUS_DATOS.dato.cua == 234)
    {
      result = true;
      //Serial.println(F("Mensaje recibido integramente ---------------"));
      //print_TUS_DATOS();
    }
    else
    {
      result = false;
      //Serial.println(F("Lectura fallida. Mensaje Incompleto ---------"));
      //print_TUS_DATOS();
    }
  }
  return result;
}
//---------------------------------------------------------------------
void borrar_MIS_DATOS()
{
  // Initzializacio de camps per transmitir als sensors
  MIS_DATOS.dato.cap           = 123; // Senyal d'inici del misatge
  MIS_DATOS.dato.sensor        = sensor;
  MIS_DATOS.dato.serie         = serie;
  MIS_DATOS.dato.valor1        = 0;
  MIS_DATOS.dato.valor2        = 0;
  MIS_DATOS.dato.valor3        = 0;
  MIS_DATOS.dato.valor4        = 0;
  MIS_DATOS.dato.valor5        = 0;
  MIS_DATOS.dato.alarma        = 0;
  MIS_DATOS.dato.interrupcio   = 0;
  MIS_DATOS.dato.epoch         = now();
  MIS_DATOS.dato.periferic     = periferic;
  MIS_DATOS.dato.menuActivo    = menuActivo;
  MIS_DATOS.dato.submenuActivo = submenuActivo;
  MIS_DATOS.dato.reservado1    = 0;
  MIS_DATOS.dato.reservado2    = 0;
  MIS_DATOS.dato.cua           = 234; // Senyal fi del misatge
}

//---------------------------------------------------------------------

void preparaRespuesta() // copiar TUS_DATOS a MIS_DATOS i preparar codis
{
  // Copiar TUS:DATOS a MIS_DATOS
  for (int i = 0 ; i < sizeof(DATOS) ; i++) MIS_DATOS.b[i] = TUS_DATOS.b[i];

  MIS_DATOS.dato.cap = 234;             // Valors invertits per indicar a control
  MIS_DATOS.dato.cua = 123;             // que es la resposta al seu missatge
  MIS_DATOS.dato.periferic = 1;
  MIS_DATOS.dato.epoch = now();         // Informar tambe del temps now()
}
//------------------------------------------------------------------

void preparaInterrupcion() // copia  // cap = 123 i cua = 234              // Es un registro ejecutivo no una respuestar TUS_DATOS a MIS_DATOS i preparar codis
{
  borrar_MIS_DATOS();
  MIS_DATOS.dato.interrupcio = 222;     // 222 --> registro alarma por interrupcion
}

//------------------------------------------------------------------

bool emitirMensaje()
{
  // Responde al mensaje recibido cambiando cap y cua y ++ el resto
  bool result = false;
  bool ok = false;
  radio.stopListening();       // Cambio de escucha a emision
  //delay(30);
  while (!ok)
  {
    for (int i = 0 ; i < 6 ; i++)                  // Reintentamos 5 veces
    {
      ok = radio.write( &MIS_DATOS, sizeof(DATOS) ); // Enviamos el mensaje
      //-------------------------------------------------------------------
      // si a la quinta iteracion no es ok
      // Ponemos "ok" a capón porque la librería NRF24L01 no funciona bien
      if (!ok)
        if (i > 4) ok = true;
    }
    //-------------------------------------------------------------------
  }
  if (ok)
  {
    result = true;
    //Serial.println(F("ok..."));                          // DEBUG
    //Serial.println(F("Respuesta enviada -----------"));  // DEBUG
    //print_MIS_DATOS();                                   // DEBUG
  }
  else
  {
    result = false;
    //Serial.println(F("NO ok..."));                             // DEBUG
    //Serial.println(F("NO Respuesta enviada --------"));        // DEBUG
  }
  // Cambio de emision a escucha
  radio.startListening();
  return result;
}

//------------------------------------------------------------------
// Rutinas de tratamientos de los sensores
//------------------------------------------------------------------

/*******************************************************************
   RUTINA:      getMoviment
   DESCRIPCIO:  interrupcio 0 (pin2)
   Aquest void fa d'interrupció per a Moviment (Activat per led al pin 2)
*****************************************************************/
void getMoviment()
{
  if (digitalRead(pinMoviment) == 1)    // (0 - no hay movimiento / 1 hay movimiento)
  {
    // Si n'hi ha moviment forçem una interrupcio - intMoviment()
    digitalWrite(pinLedMoviment, HIGH);
    espera(500);
    digitalWrite(pinLedMoviment, LOW);

  }
}

/*******************************************************************
   RUTINA:      intMoviment
   DESCRIPCIO:  interrupcio 0 (pin2)
   Aquest void executa una interrupció 0 al pin 2
 ******************************************************************/

void intMoviment()
{
  // Hi ha hagut una interrupció a la linia de PIR controlada per aquest nodo
  // Se envia registro de RF correspondienteval

  // encendre un led per visualitzar l'alarma
  // aqui se puede encender un led como señal de alarma
  // Limitem el numero d'alarmas al limit prefixat "alarmas"
  if (contaMoviment < alarmas)
  {
    preparaRespuesta();
    preparaInterrupcion();
    emitirMensaje();
    
    contaInfrarojos ++;                // demani SMS d'avis al block de comunicacions
    Serial.println(F("ok..."));                          // DEBUG
    Serial.println(F("Respuesta alarma enviada -----------"));  // DEBUG
    print_MIS_DATOS();                                   // DEBUG
  }
}

//------------------------------------------------------------------

void sincronizaArduino() // Sincronizar el reloj al primer registro de sincronia
{
  ahora = now();
  if ((!estaEnHora) || !( ahora > 1000000)) // Si el temps encara no es ajustat
  {
    setTime(TUS_DATOS.dato.epoch);
    ahora = now();
    estaEnHora = true;
    // Ademas de sincronizar el reloj, recojemos información sobre menu activo
    menuActivo = TUS_DATOS.dato.menuActivo;
    submenuActivo = TUS_DATOS.dato.submenuActivo;
  }
}

//---------------------------------------------------------------------

void print_TUS_DATOS() // Debug
{
  time_t t = now();
  printFecha(t);
  Serial.print(F("cap             = ")); Serial.println(TUS_DATOS.dato.cap);
  Serial.print(F("sensor          = ")); Serial.println(TUS_DATOS.dato.sensor);
  Serial.print(F("serie           = ")); Serial.println(TUS_DATOS.dato.serie);
  Serial.print(F("valor sensor 1  = ")); Serial.println(TUS_DATOS.dato.valor1);
  Serial.print(F("valor sensor 2  = ")); Serial.println(TUS_DATOS.dato.valor2);
  Serial.print(F("valor sensor 3  = ")); Serial.println(TUS_DATOS.dato.valor3);
  Serial.print(F("valor sensor 4  = ")); Serial.println(TUS_DATOS.dato.valor4);
  Serial.print(F("valor sensor 5  = ")); Serial.println(TUS_DATOS.dato.valor5);
  Serial.print(F("alarma          = ")); Serial.println(TUS_DATOS.dato.alarma);
  Serial.print(F("interrupcio     = ")); Serial.println(TUS_DATOS.dato.interrupcio);
  Serial.print(F("epoch           = ")); Serial.println(TUS_DATOS.dato.epoch);
  Serial.print(F("periferic       = ")); Serial.println(TUS_DATOS.dato.periferic);
  Serial.print(F("menuActivo      = ")); Serial.println(TUS_DATOS.dato.menuActivo);
  Serial.print(F("submenuActivo   = ")); Serial.println(TUS_DATOS.dato.submenuActivo);
  Serial.print(F("reservado1      = ")); Serial.println(TUS_DATOS.dato.reservado1);
  Serial.print(F("reservado2      = ")); Serial.println(TUS_DATOS.dato.reservado2);
  Serial.print(F("cua             = ")); Serial.println(TUS_DATOS.dato.cua);
  Serial.println(F("================================================="));
}

//---------------------------------------------------------------------

void print_MIS_DATOS() // Debug
{
  time_t t = now();
  printFecha(t);
  Serial.print(F("cap             = ")); Serial.println(MIS_DATOS.dato.cap);
  Serial.print(F("sensor          = ")); Serial.println(MIS_DATOS.dato.sensor);
  Serial.print(F("serie           = ")); Serial.println(MIS_DATOS.dato.serie);
  Serial.print(F("valor sensor 1  = ")); Serial.println(MIS_DATOS.dato.valor1);
  Serial.print(F("valor sensor 2  = ")); Serial.println(MIS_DATOS.dato.valor2);
  Serial.print(F("valor sensor 3  = ")); Serial.println(MIS_DATOS.dato.valor3);
  Serial.print(F("valor sensor 4  = ")); Serial.println(MIS_DATOS.dato.valor4);
  Serial.print(F("valor sensor 5  = ")); Serial.println(MIS_DATOS.dato.valor5);
  Serial.print(F("alarma          = ")); Serial.println(MIS_DATOS.dato.alarma);
  Serial.print(F("interrupcio     = ")); Serial.println(MIS_DATOS.dato.interrupcio);
  Serial.print(F("epoch           = ")); Serial.println(MIS_DATOS.dato.epoch);
  Serial.print(F("periferic       = ")); Serial.println(MIS_DATOS.dato.periferic);
  Serial.print(F("menuActivo      = ")); Serial.println(MIS_DATOS.dato.menuActivo);
  Serial.print(F("submenuActivo   = ")); Serial.println(MIS_DATOS.dato.submenuActivo);
  Serial.print(F("reservado1      = ")); Serial.println(MIS_DATOS.dato.reservado1);
  Serial.print(F("reservado2      = ")); Serial.println(MIS_DATOS.dato.reservado2);
  Serial.print(F("cua             = ")); Serial.println(MIS_DATOS.dato.cua);
  Serial.println(F("================================================="));
}

//---------------------------------------------------------------------

void printFecha(time_t  t) // Debug
{
  Serial.print(F("Fecha y Hora (D/M/A H;M;S) -->  "));
  Serial.print(day(t)) ;    Serial.print(+ "/") ;   Serial.print(month(t));   Serial.print(+ "/") ;
  Serial.print(year(t));    Serial.print( " ") ;
  Serial.print(hour(t));   Serial.print(+ ":") ;   Serial.print(minute(t));   Serial.print(F(":")) ;
  Serial.println(second(t));
  //espera(500);
}

//---------------------------------------------------------------------

void espera(int t)
{
  // Rutina per a substituir delays no desitjats per tal d'utilitzar Interrupcións
  // NOTA: No utilitzar dins del codigo d'una Interrupció
  t = t - ((t / 1000) * 1.7); // Compensacio per l'increment de temps observat sobre "delay();"
  int coci = t / 1000; // Numero de segons sencers
  int rest = t % 1000; // Milisegons restants
  /* // Debug
    Serial.print(F("t = "));
    Serial.println(t);
    // Debug */
  while (coci > 0)
  {
    unsigned long tempsini = millis();
    unsigned long tempsfin = tempsini;
    while ((tempsfin - tempsini) <= 1000) // loop de 1000ms
    {
      tempsfin = millis();
    }
    coci = coci - 1;
  }
}
// Fin del codigo
////////////////////////////////////////////////////////////////////////////////////////////
